@@ -7,7 +7,7 @@
 #define RF_nativeNaN NA_REAL
 #define RF_nativeIsNaN ISNA
 
-
+ 
 #include <stdio.h>
 #include <stdarg.h>
 #include <stdlib.h>
@@ -108,10 +108,9 @@ typedef unsigned long ulong;
 #define OPT_VARUSED_F 0x00001000 
 #define OPT_VARUSED_T 0x00002000 
 #define OPT_PERF_GMN2 0x00004000 
-#define OPT_PERF_GMND 0x00008000 
+#define OPT_PERF_GMNQ 0x00008000 
 #define OPT_IMPU_ONLY 0x00010000 
 #define OPT_OUTC_TYPE 0x00020000 
-#define OPT_SPLT_NULL 0x00040000 
 #define OPT_BOOT_TYP1 0x00080000 
 #define OPT_BOOT_TYP2 0x00100000 
 #define OPT_COMP_RISK 0x00200000 
@@ -176,7 +175,8 @@ typedef unsigned long ulong;
 #define MVMX_SPLIT  15 
 #define CUST_SPLIT  16
 #define SURV_L2IMP  17
-#define MAXM_SPLIT  18 
+#define CLAS_RNKPS  18
+#define MAXM_SPLIT  19 
 #define SPLIT_MIA_NONE  0  
 #define SPLIT_MIA_LEFT  1  
 #define SPLIT_MIA_RGHT  2  
@@ -194,9 +194,9 @@ typedef unsigned long ulong;
 #define RF_WGHT_INTEGER 2
 #define RF_WGHT_GENERIC 3
 #define RF_DISTANCE_EUCLIDEAN 1
-#define NATIVE_TYPE_NUMERIC   0
+#define NATIVE_TYPE_CHARACTER 0
 #define NATIVE_TYPE_INTEGER   1
-#define NATIVE_TYPE_CHARACTER 2
+#define NATIVE_TYPE_NUMERIC   2
 typedef struct node Node;
 struct node {
   struct node *parent;
@@ -311,7 +311,7 @@ double getGMeanIndex(uint    size,
                      double *responsePtr,
                      uint   *denomCount,
                      double *maxVote);
-double getGMeanDRC(uint     size,
+double getGMeanQC(uint     size,
                    uint     rTarget,
                    double  *responsePtr,
                    double **outcomeCLS,
@@ -336,11 +336,11 @@ Factor *makeFactor(uint r, char bookFlag);
 void free_Factor(Factor *f);
 char bookFactor(Factor *f);
 char unbookFactor(Factor *f);
-void bookPair (uint    levelCount, 
-               uint    setSize, 
-               uint    setColumn, 
-               uint   *setRow, 
-               uint   *pair, 
+void bookPair (uint    levelCount,
+               uint    groupIndex,
+               uint    levelIndex,
+               uint   *row,
+               uint   *level,
                Factor *f);
 void nChooseK (uint n, uint r, char type, void *result);
 char reduceFraction(uint *numerator, uint *denominator);
@@ -714,7 +714,7 @@ void updateCDF(uint    treeID,
                uint   *sampleSize,
                uint    sampleSlot,
                double *cdf,
-               uint   *cdfSize,
+               uint    cdfSize,
                uint   *density,
                uint   *densitySize,
                uint  **densitySwap,
@@ -750,9 +750,9 @@ struct snpAuxiliaryInfo {
   uint identity;
   ulong linearSize;
   void *snpPtr;
-  void *auxiliaryPtr;
+  void *auxiliaryArrayPtr;
   uint dimSize;
-  uint *dim;
+  int *dim;
 };
 char getBestSplit(uint    treeID,
                   Node   *parent,
@@ -812,6 +812,20 @@ char classificationXwghtSplit(uint    treeID,
                               char  **splitIndicator,
                               char   *splitMIA,
                               char    multImpFlag);
+char classificationRankedPS(uint    treeID,
+                            Node   *parent,
+                            uint   *repMembrIndx,
+                            uint    repMembrSize,
+                            uint   *allMembrIndx,
+                            uint    allMembrSize,
+                            uint   *splitParameterMax,
+                            double *splitValueMaxCont,
+                            uint   *splitValueMaxFactSize,
+                            uint  **splitValueMaxFactPtr,
+                            double *splitStatistic,
+                            char  **splitIndicator,
+                            char   *splitMIA,
+                            char    multImpFlag);
 char regressionXwghtSplit(uint    treeID,
                           Node   *parent,
                           uint   *repMembrIndx,
@@ -826,6 +840,74 @@ char regressionXwghtSplit(uint    treeID,
                           char  **splitIndicator,
                           char   *splitMIA,
                           char    multImpFlag);
+char regressionXwghtSplitMIA (uint    treeID,
+                              Node   *parent,
+                              uint   *repMembrIndx,
+                              uint    repMembrSize,
+                              uint   *allMembrIndx,
+                              uint    allMembrSize,
+                              uint   *splitParameterMax,
+                              double *splitValueMaxCont,
+                              uint   *splitValueMaxFactSize,
+                              uint  **splitValueMaxFactPtr,
+                              double *splitStatistic,
+                              char  **splitIndicator,
+                              char   *splitMIA,
+                              char    multImpFlag);
+char regressionXwghtSplitGreedy(uint    treeID,
+                          Node   *parent,
+                          uint   *repMembrIndx,
+                          uint    repMembrSize,
+                          uint   *allMembrIndx,
+                          uint    allMembrSize,
+                          uint   *splitParameterMax,
+                          double *splitValueMaxCont,
+                          uint   *splitValueMaxFactSize,
+                          uint  **splitValueMaxFactPtr,
+                          double *splitStatistic,
+                          char  **splitIndicator,
+                          char   *splitMIA,
+                          char    multImpFlag);
+char selectRandomCovariatesGreedy(uint     treeID,
+                                  Node     *parent,
+                                  uint     *repMembrIndx,
+                                  uint      repMembrSize,
+                                  uint     *covariateIndex,
+                                  uint     *uniformCovariateSize,
+                                  uint     *uniformCovariateIndex,
+                                  double   *cdf,
+                                  uint     *cdfSize,
+                                  uint     *cdfSort,
+                                  uint     *density,
+                                  uint     *densitySize,
+                                  uint    **densitySwap,
+                                  uint     *covariate,
+                                  uint     *actualCovariateCount,
+                                  uint     *candidateCovariateCount,
+                                  double   *splitVector,
+                                  uint     *splitVectorSize,
+                                  uint    **indxx,
+                                  uint      nonMissMembrSizeStatic,
+                                  uint     *nonMissMembrIndxStatic,
+                                  uint     *nonMissMembrSize,
+                                  uint    **nonMissMembrIndx,
+                                  char      multImpFlag);
+void unselectRandomCovariatesGreedy(uint      treeID,
+                              Node     *parent,
+                              uint      repMembrSize,
+                              uint     *indxx,
+                              uint     nonMissMembrSizeStatic,
+                              uint    *nonMissMembrIndx,
+                                    char      multImpFlag);
+typedef struct hyperCube HyperCube;
+struct hyperCube {
+  unsigned int r; 
+  unsigned int cardinalGroupCount; 
+  uint *complementaryPairCount;
+  void *cardinalGroupSize; 
+  unsigned int **cardinalGroupBinary;
+  unsigned int mwcpSize;
+};
 char logRankNCR(uint    treeID,
                 Node   *parent,
                 uint   *repMembrIndx,
@@ -1097,7 +1179,7 @@ void unselectRandomCovariates(uint      treeID,
                               uint      nonMissMembrSizeStatic,
                               uint     *nonMissMembrIndx,
                               char      multImpFlag);
-char selectRandomCovariatesNew(uint     treeID,
+char selectRandomCovariatesMIA(uint     treeID,
                             Node     *parent,
                             uint     *repMembrIndx,
                             uint      repMembrSize,
@@ -1123,14 +1205,14 @@ char selectRandomCovariatesNew(uint     treeID,
                             uint     *missMembrSize,
                             uint    **missMembrIndx,
                             char      multImpFlag);
-void unselectRandomCovariatesNew(uint      treeID,
-                              Node     *parent,
-                              uint      repMembrSize,
-                              uint     *indxx,
-                              uint      nonMissMembrSizeStatic,
-                              uint     *nonMissMembrIndx,
+void unselectRandomCovariatesMIA(uint      treeID,
+                                 Node     *parent,
+                                 uint      repMembrSize,
+                                 uint     *indxx,
+                                 uint      nonMissMembrSizeStatic,
+                                 uint     *nonMissMembrIndx,
                                  uint     *missMembrIndx,
-                              char      multImpFlag);
+                                 char      multImpFlag);
 uint virtuallySplitNode(uint  treeID,
                            char  factorFlag,
                            uint  mwcpSizeAbsolute,
@@ -1180,7 +1262,7 @@ char getPreSplitResult (uint      treeID,
                         double   *preSplitMean,
                         char      multImpFlag,
                         char      multVarFlag);
-char getPreSplitResultNew (uint      treeID,
+char getPreSplitResultMIA (uint      treeID,
                            Node     *parent,
                            uint      repMembrSize,
                            uint     *repMembrIndx,
@@ -1198,7 +1280,7 @@ void unstackPreSplit (char      preliminaryResult,
                       uint      repMembrSize,
                       double   *splitVector,
                       uint     *nonMissMembrIndx);
-void unstackPreSplitNew (char      preliminaryResult,
+void unstackPreSplitMIA (char      preliminaryResult,
                          char      multImpFlag,
                          char      multVarFlag,
                          uint      repMembrSize,
@@ -1222,7 +1304,7 @@ char updateMaximumSplit(uint    treeID,
                         uint  **splitValueMaxFactPtr,
                         void   *splitVectorPtr,
                         char  **splitIndicator);
-char updateMaximumSplitNew(uint    treeID,
+char updateMaximumSplitMIA(uint    treeID,
                            Node   *parent,
                            double  delta,
                            uint    candidateCovariateCount,
@@ -1244,7 +1326,8 @@ char updateMaximumSplitNew(uint    treeID,
 void updateNodeStatistics(Node *parent, double delta, uint candidateCovariateCount, uint covariate);
 void initializeTimeArrays(char mode);
 void stackFactorArrays();
-void stackFactorGeneric(uint    size,
+void stackFactorGeneric(char    respFlag,
+                        uint    size,
                         char   *type,
                         uint  **p_factorMap,
                         uint   *factorCount,
@@ -1307,6 +1390,7 @@ void stackDefinedOutputObjects(char      mode,
                                Node   ***pRF_root,
                                uint    **pRF_tLeafCount_,
                                double  **pRF_proximity_,
+                                
                                double  **pRF_weight_,
                                int     **pRF_seed_,
                                double  **p_imputation_,
@@ -1337,12 +1421,6 @@ void stackTNQualitativeOutputObjects(char     mode,
                                      uint   **pRF_AMBR_ID_,
                                      uint   **pRF_TN_RCNT_,
                                      uint   **pRF_TN_ACNT_);
-void stackAuxTNQualitativeStructures(char     mode,
-                                     uint    *pRF_RMBR_ID_,
-                                     uint    *pRF_AMBR_ID_,
-                                     uint    *pRF_TN_RCNT_,
-                                     uint    *pRF_TN_ACNT_);
-void unstackAuxTNQualitativeStructures(char    mode);
 void stackTNQuantitativeOutputObjects(char     mode,
                                       double **pRF_TN_SURV_,
                                       double **pRF_TN_MORT_,
@@ -1351,26 +1429,18 @@ void stackTNQuantitativeOutputObjects(char     mode,
                                       double **pRF_TN_CIFN_,
                                       double **pRF_TN_REGR_,
                                       uint   **pRF_TN_CLAS_);
-void stackAuxTNQuantitativeStructures(char    mode,
-                                      double *pRF_TN_SURV_,
-                                      double *pRF_TN_MORT_,
-                                      double *pRF_TN_NLSN_,
-                                      double *pRF_TN_CSHZ_,
-                                      double *pRF_TN_CIFN_,
-                                      double *pRF_TN_REGR_,
-                                      uint   *pRF_TN_CLAS_);
-void unstackAuxTNQuantitativeStructures(char    mode);
 void verifyAndRegisterCustomSplitRules();
 extern void registerCustomFunctions();
 void stackAuxiliaryInfoList();
 void allocateAuxiliaryInfo(char   type,
                            uint   identity,
-                           ulong  linearSize,
                            void  *snpPtr,
-                           void  *auxiliaryPtr,
+                           void  *auxiliaryArrayPtr,
                            uint   dimSize,
-                           uint  *dim);
+                           int   *dim);
+uint getAuxDim(int *dim, uint preIndex, uint postIndex);
 void unstackAuxiliaryInfoAndList();
+void memoryCheck();
 void stackIncomingResponseArrays(char mode);
 void unstackIncomingResponseArrays(char mode);
 void unstackIncomingCovariateArrays(char mode);
@@ -1492,63 +1562,10 @@ void unstackMultiClassProb(Terminal *tTerm);
 void stackMeanResponse(Terminal *tTerm, unsigned int rnfCount);
 void unstackMeanResponse(Terminal *tTerm);
 void getTerminalInfo(Terminal *termPtr);
-#include <stdlib.h>
-#include <time.h>
-#ifndef TRUE
-#define TRUE      0x01
-#endif
-#ifndef FALSE
-#define FALSE     0x00
-#endif
-
-#include <R_ext/Print.h>
-#ifndef RF_nativePrint
-#define RF_nativePrint printR
-#endif
-extern void printR(char *format, ...);
-
-
-#define SUMM_DEF_TRACE  0x00000001
-#define SUMM_LOW_TRACE  0x00000002
-#define SUMM_MED_TRACE  0x00000004
-#define SUMM_HGH_TRACE  0x00000008
-#define SPLT_DEF_TRACE  0x00000010
-#define SPLT_LOW_TRACE  0x00000020
-#define SPLT_MED_TRACE  0x00000040
-#define SPLT_HGH_TRACE  0x00000080
-#define FORK_DEF_TRACE  0x00000100
-#define MISS_LOW_TRACE  0x00000200
-#define MISS_MED_TRACE  0x00000400
-#define MISS_HGH_TRACE  0x00000800
-#define OUTP_DEF_TRACE  0x00001000
-#define NUMR_DEF_TRACE  0x00002000
-#define FACT_LOW_TRACE  0x00004000
-#define FACT_HGH_TRACE  0x00008000
-#define ENSB_LOW_TRACE  0x00010000
-#define ENSB_HGH_TRACE  0x00020000
-#define BOOT_MED_TRACE  0x00040000
-#define VIMP_LOW_TRACE  0x00080000
-#define NODE_DEF_TRACE  0x00100000
-#define TIME_DEF_TRACE  0x00200000
-#define RAND_DEF_TRACE  0x00400000
-#define TURN_OFF_TRACE  0x00000000
-void setTraceFlag(unsigned int traceFlag, unsigned int tree);
-unsigned int getTraceFlag(unsigned int tree);
-unsigned int updateTimeStamp(unsigned int before);
-unsigned int getNodeDefTraceFlag();
-unsigned int getForkDefTraceFlag();
-unsigned int getTurnOffTraceFlag();
-unsigned int getNumrDefTraceFlag();
-unsigned int getTimeDefTraceFlag();
-void   setMaxMemoryAllocation(size_t value);
-void   setMinMemoryAllocation(size_t value);
-size_t getMaxMemoryAllocation();
-size_t getMinMemoryAllocation();
-void   increaseMemoryAllocation(size_t amount);
-void   decreaseMemoryAllocation(size_t amount);
 void acquireTree(char mode, uint r, uint b);
 void updateWeight(char mode, uint b);
 void finalizeWeight(char mode);
+ 
 void updateProximity(char mode, uint b);
 void finalizeProximity(char mode);
 void updateSplitDepth(uint treeID, Node *rootPtr, uint maxDepth);
@@ -1684,7 +1701,7 @@ SEXP rfsrcGrow(SEXP traceFlag,
                SEXP bootstrapSize,
                SEXP bootstrap,
                SEXP caseWeight,
-               SEXP xSplitStatWt,
+               SEXP xSplitStatWeight,
                SEXP yWeight,
                SEXP xWeight,
                SEXP xData,
@@ -1731,6 +1748,7 @@ SEXP rfsrcPredict(SEXP traceFlag,
                   SEXP intrPredictor,
                   SEXP sobservationSize,
                   SEXP sobservationIndv,
+                                  
                   SEXP partialType,
                   SEXP partialXvar,
                   SEXP partialLength,
@@ -1752,7 +1770,7 @@ SEXP rfsrcPredict(SEXP traceFlag,
 void exit2R();
 void printR(char *format, ...);
 void setNativeGlobalEnv();
-void *copy1DObject(SEXP arr, char type, uint size);
+void *copy1DObject(SEXP arr, char type, uint size, char actual);
 void *copy2DObject(SEXP arr, char type, char flag, uint row, uint col);
 void free_1DObject(void *arr, char type, uint size);
 void free_2DObject(void *arr, char type, char flag, uint row, uint col);
@@ -1761,6 +1779,7 @@ void *stackAndProtect(uint  *sexpIndex,
                       char   sexpType,
                       uint   sexpIdentity,
                       ulong  size,
+                      double value,
                       char **sexpString,
                       void  *auxiliaryPtr,
                       uint   auxiliaryDimSize,
